#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#else
#include <strings.h>
extern "C" void exit(...);
#endif

#include  "tof77.h"

#include "compilerApi.h"


void forall_do(SgStatement *s)
{
	SgForallStmt *forall = isSgForallStmt(s);
	ASSERT(forall);
printf("Serialize Forall Statement to Do-loop.\n");
	SgStatement *body = s->childList1(0);
	SgStatement *do_body = body->copyPtr();
	SgExpression *mask = forall->getMask();
	SgStatement *newdo;
	if (mask)
		newdo = new SgLogIfStmt(*mask, *do_body);
	else
		newdo = do_body;
	for (int i=0; i<forall->indexNumber(); i++)
	{
		SgExpression *index_op = forall->getIndex(i);
	  	SgSymbol *index = forall->getIndexName(i);
		SgTripletExp *ddot = (SgTripletExp*)index_op->rhs();
		ASSERT(ddot);
		SgExpression *start = ddot->item(0);
		SgExpression *end = ddot->item(1);
		SgExpression *stride = ddot->item(2);
		newdo = new SgForStmt(index, start, end, stride, newdo);	
	}
	s->insertStmtAfter(*newdo, *s->controlParent());
	s->deleteStmt();
}

void proc_dummy(SgStatement *s)
{
	SgCallStmt * call = (SgCallStmt *)s;
	for (int i=0; i<call->numberOfArgs(); i++) {
	    SgExpression * var = call->arg(i);
	    if (isSgArrayType(var->type()) && var->variant() == ARRAY_REF)
	    {
	    }
	}
}

void analyse6(SgStatement * s)
{
  switch(s->variant())
  {
    case GLOBAL:
      {
         int org_numberOfChildren=s->numberOfChildrenList1();
         SgStatement * * children=new SgStatement * [org_numberOfChildren];

         int i;

         for(i=0;i<org_numberOfChildren;i++)
             children[i]=s->childList1(i);
         for(i=0;i<org_numberOfChildren;i++)
             analyse6(children[i]);

        delete [] children;

        break;
      };

   case PROG_HEDR:
      { 
         int org_numberOfChildren=s->numberOfChildrenList1();
         SgStatement * * children=new SgStatement * [org_numberOfChildren];

         int i;

         for(i=0;i<org_numberOfChildren;i++)
             children[i]=s->childList1(i); 

         in_main_program=on;

         for(i=0;i<org_numberOfChildren;i++)
             analyse6(children[i]);

         delete [] children;

         in_main_program=off;
  
         break;
       };

   case PROC_HEDR:
      {
        SgProcHedrStmt * prochedr=(SgProcHedrStmt *)s;
        int org_numberOfChildren=s->numberOfChildrenList1();
        SgStatement * * children=new SgStatement * [org_numberOfChildren];

        int i;

        for(i=0;i<org_numberOfChildren;i++)
            children[i]=s->childList1(i);

        in_procedure=on;

        for(i=0;i<org_numberOfChildren;i++)
             analyse6(children[i]);
     
        delete [] children;

        in_procedure=off;

        break;
      };

    case FUNC_HEDR:
      {
         SgFuncHedrStmt * prochedr=(SgFuncHedrStmt *)s;
         int org_numberOfChildren=s->numberOfChildrenList1();
         SgStatement * * children=new SgStatement * [org_numberOfChildren];

         int i;

         for(i=0;i<org_numberOfChildren;i++)
              children[i]=s->childList1(i);

          in_function=on;

         for(i=0;i<org_numberOfChildren;i++)
             analyse6(children[i]);

         delete [] children;

         in_function=off;

         break;
      };  
  case FOR_NODE:
  case LOGIF_NODE:
  case ELSEIF_NODE:
  case LOGIF_STMT:
  case SWITCH_NODE:
  case CASE_NODE:
    {
      int org_numberOfChildren1=s->numberOfChildrenList1();

      SgStatement * * children1 = new SgStatement * [org_numberOfChildren1];

      int org_numberOfChildren2=s->numberOfChildrenList2();

      SgStatement * * children2 = new SgStatement * [org_numberOfChildren2];

      int i;

      for (i=0; i<org_numberOfChildren1; i++)
	children1[i]=s->childList1(i);

      for (i=0; i<org_numberOfChildren1; i++)
	analyse6(children1[i]);
      
      delete [] children1;

      for (i=0; i<org_numberOfChildren2; i++)
	children2[i]=s->childList2(i);			

      for (i=0; i<org_numberOfChildren2; i++)
	analyse6(children2[i]);
      
      delete [] children2;

      break;

    }
   case FORALL_STMT:
      {
	forall_do(s);
	break;
      };
   case PROC_STAT:
      {
	proc_dummy(s);
	break;
      }
   case DISTRIBUTE_DECL:
   case ALIGN_DECL:
   case PROCESSORS_STMT:
   case TEMPLATE_STMT:
   case INHERIT_DECL:
   case INDEPENDENT_STMT:
   case INTERFACE_STMT:
   case MEMORY_DECL:
   case OUTOFCORE_DECL:
   case INCORESIZE_DECL:
      {
	s->deleteStmt();
	break;
      };
  }
 
}
